﻿using System.Security;
using System.Security.Permissions;
using System.Security.Principal;
using System.Web.UI;

namespace Mindsharp.Utilities
{
  // Demand that the immediate calling assembly to have a custom CAS permission
  [PrivilegedSandboxPermissionAttribute(SecurityAction.LinkDemand)]
  public class PrivilegedSandbox
  {
    public Control LoadControl(string virtualPath)
    {
      Control returnControl = default(Control);

      try
      {
        UserControl templateControl = new UserControl();

        // Assert that the following code will use Full Trust
        // regardless of whether the assemblies in the call stack
        // have been granted Full Trust or not
        // This assembly must be granted Full Trust in order for it to assert
        // that level of CAS on behalf of the callers
        PermissionSet fullTrust = new PermissionSet(PermissionState.Unrestricted);
        fullTrust.Assert();

        WindowsImpersonationContext wic = default(WindowsImpersonationContext);
        try
        {
          // Impersonate the Machine System Account
          // This account will always have access to the ASCX file on the C: drive
          // even if this code is running under a non-windows identity like:
          // Anonymous, FBA, etc.
          wic = WindowsIdentity.Impersonate(System.IntPtr.Zero);

          // Load the User Control specified in the virtual path
          // Using the current page
          // The ASP.NET SDK is wrong, this statement requires Full Trust
          // LoadControl > GetVPathBuildResult > if (HttpRuntime.IsFullTrust)
          returnControl = templateControl.LoadControl(virtualPath);
        }
        finally
        {
          // Return to the initial security context
          wic.Undo();
        }
      }
      finally
      {
        // Return to the initial CAS context
        CodeAccessPermission.RevertAssert();
      }

      return returnControl;
    }
  }
}
