﻿using System.Security;
using System.Security.Permissions;

namespace Mindsharp.Utilities
{
  // Specify that only a single instance of this attribute is allowed
  // and only applied at a class level
  // While arbitrary, these settings provide some guidance as to the use of this attribute
  [System.AttributeUsage(System.AttributeTargets.Class, AllowMultiple=false)]
  public class PrivilegedSandboxPermissionAttribute :
    System.Security.Permissions.CodeAccessSecurityAttribute
  {
    public new bool Unrestricted { get; set; }

    // Required ctor
    public PrivilegedSandboxPermissionAttribute(SecurityAction action)
      : base(action) { }

    // Use the value for Unrestricted to determine what the permission will initially look like
    // Presumably, Unrestricted would initially always be true
    public override IPermission CreatePermission()
    {
      // If the boolean is true
      if (Unrestricted)
      {
        // Use the Unrestricted Enum in the constructor
        return new PrivilegedSandboxPermission(PermissionState.Unrestricted);
      }
      else
      {
        return new PrivilegedSandboxPermission(PermissionState.None);
      }
    }
  }
}
